/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2012, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

//13/12/12 17:00 NDP : AwakeDataSourceCreator class creation

package org.awakefw.commons.server.util;

import java.io.File;
import java.util.Properties;

import javax.sql.DataSource;

import org.awakefw.commons.api.server.AwakeDataSource;
import org.awakefw.commons.server.util.embed.JdbcDriverConstants;
import org.awakefw.file.api.util.AwakeDebug;
import org.awakefw.file.util.Tag;

/**
 * 
 * Creates an Awake Datasource form values stores in a Properties
 * 
 * @author Nicolas de Pomereu
 *
 */
@SuppressWarnings("deprecation")
public class AwakeDataSourceCreator {

    private static boolean DEBUG = AwakeDebug.isSet(AwakeDataSourceCreator.class);
    
    /** The properties that contain all info for the AwajeDataSource to create */
    private Properties properties = null;
    
    /** The base error message if user configuration fails */
    private String baseConfigErrorMessage = null;

        
    /**
     * Constructor
     * 
     * @param properties		the properties that contain the JDBC Driver info & pool configuration
     * @param baseConfigErrorMessage	the prefix error message to display if user configuration failure
     */
    public AwakeDataSourceCreator(Properties properties, String baseConfigErrorMessage) {

	if (properties == null) {
	    throw new IllegalArgumentException("properties can not be null!");
	}
	
	if (baseConfigErrorMessage == null) {
	    throw new IllegalArgumentException("baseConfigErrorMessage can not be null!");
	}	
	
	this.properties = properties;
	this.baseConfigErrorMessage = baseConfigErrorMessage;
    }

    
    /**
     * Creates a an AwakeDataSource instance (with the passed JNDI properties)
     * @param name
     *            The name of this object relating to <code>nameCtx</code>
     */
    public DataSource createAwakeDataSource() throws Exception {

	String driverClassName = properties.getProperty(JdbcDriverConstants.DRIVER_CLASS_NAME);
	String url = properties.getProperty(JdbcDriverConstants.URL);
	String username = properties.getProperty(JdbcDriverConstants.USERNAME);
	String password = properties.getProperty(JdbcDriverConstants.PASSWORD);
	String minConnsStr = properties.getProperty(JdbcDriverConstants.MIN_CONNS);
	String maxConnsStr = properties.getProperty(JdbcDriverConstants.MAX_CONNS);
	String maxConnTimeStr = properties.getProperty(JdbcDriverConstants.MAX_CONN_TIME);
	String maxCheckoutSecondsStr = properties.getProperty(JdbcDriverConstants.MAXCHECKOUTSECONDS);
	String logAppendStr = properties.getProperty(JdbcDriverConstants.LOG_APPEND);
	String logFileStr = properties.getProperty(JdbcDriverConstants.LOG_FILE);

	debug("In createAwakeDataSource():");
	debug("driverClassName: " + driverClassName);
	debug("url                : " + url);
	debug("username           : " + username);
	debug("password           : " + password);
	debug("minConnsStr        : " + minConnsStr);
	debug("maxConnsStr        : " + maxConnsStr);
	debug("maxConnTimeStr     : " + maxConnTimeStr);
	debug("maxCheckoutSeconds : " + maxCheckoutSecondsStr);
	debug("logAppendStr       : " + logAppendStr);	
	debug("logFileStr         : " + logFileStr);

	int minConns = 0;
	int maxConns = 0;
	double maxConnTime = 0;
	int maxCheckoutSeconds = 0;
	boolean logAppend = false;
	
	
	try {
	    minConns = Integer.parseInt(minConnsStr);
	} catch (NumberFormatException e) {
	    throw new IllegalArgumentException(
		    Tag.AWAKE_USER_CONFIG_FAIL
		    	    + this.baseConfigErrorMessage +
			    " Parameter minConns is not numeric: "
			    + minConnsStr);
	}

	try {
	    maxConns = Integer.parseInt(maxConnsStr);
	} catch (NumberFormatException e) {
	    throw new IllegalArgumentException(
		    Tag.AWAKE_USER_CONFIG_FAIL
	    	    + this.baseConfigErrorMessage +
			    " Parameter maxConns is not numeric: "
			    + maxConnsStr);
	}

	if (maxConnTimeStr == null || maxConnTimeStr.isEmpty()) {
	    maxConnTime = -1;
	} else {
	    try {
		maxConnTime = Double.parseDouble(maxConnTimeStr);
	    } catch (NumberFormatException e) {
		throw new IllegalArgumentException(
			Tag.AWAKE_USER_CONFIG_FAIL
		    	    + this.baseConfigErrorMessage +
				" Parameter maxConnTimeStr is not numeric: "
				+ maxConnTimeStr);
	    }
	}
		
	if (maxCheckoutSecondsStr == null || maxCheckoutSecondsStr.isEmpty()) {
	    maxCheckoutSeconds = -1;
	} else {
	    try {
		maxCheckoutSeconds = Integer.parseInt(maxCheckoutSecondsStr);
	    } catch (NumberFormatException e) {
		throw new IllegalArgumentException(
			Tag.AWAKE_USER_CONFIG_FAIL
		    	    + this.baseConfigErrorMessage +
				" Parameter maxCheckoutSecondsStr is not numeric: "
				+ maxCheckoutSecondsStr);
	    }
	}
	
	if (logAppendStr == null || logAppendStr.isEmpty()) {
	    logAppend = false;
	} else {
	    try {
		logAppend = Boolean.parseBoolean(logAppendStr);
	    } catch (NumberFormatException e) {
		throw new IllegalArgumentException(
			Tag.AWAKE_USER_CONFIG_FAIL
		    	    + this.baseConfigErrorMessage +
				" Parameter maxCheckoutSecondsStr is not numeric: "
				+ maxCheckoutSecondsStr);
	    }
	}
	
	
	// This is an AwakeDataSource!
	AwakeDataSource awakeDataSource = new AwakeDataSource();
	awakeDataSource.setDriverClassName(driverClassName);
	awakeDataSource.setUrl(url);
	awakeDataSource.setUsername(username);
	awakeDataSource.setPassword(password);
	awakeDataSource.setMinConns(minConns);
	awakeDataSource.setMaxConns(maxConns);
	
	// Non mandatory value
	if (maxConnTime != -1) {
	    awakeDataSource.setMaxConnTime(maxConnTime);
	}
	
	// Non mandatory value
	if (maxCheckoutSeconds != -1) {
	    awakeDataSource.setMaxCheckoutSeconds(maxCheckoutSeconds);
	}	
	
	// Non mandatory value, but no need to make a test
	awakeDataSource.setLogAppend(logAppend);

	if (logFileStr != null && !logFileStr.isEmpty()) {
	    awakeDataSource.setLogFile(new File(logFileStr));
	}

	// Return the configured AwakeDataSource instance!
	return awakeDataSource;
    }

    



    /**
     * debug tool
     */
    private void debug(String s) {
	if (DEBUG) {
	    AwakeServerLogger.log(s);
	}
    }    
    
}
